==> org-lsst-ccs-subsystem-fcs-main-5.0.4-tests.jar: org/lsst/ccs/subsystems/fcs/scripts/loaderClampCarrierTest.py <==
#####################################################################
# To test sequence of actions on loader :
# goToHandoff, close, clamp, goToStorage, goToHandoff, unclamp, open
# Tested on loader proto at LPSC in May 2017
# In scripts directory (org-lsst-ccs-subsystem-fcs/main/scripts)
# this script's name is loaderClampCarrierTest.py
#####################################################################
### change iteration number / nombre d'iterations a changer si besoin est
iter = 2
#####################################################################

#import CCS Scripting Implementation                                                                                          
from org.lsst.ccs.scripting import CCS
#import the definition for a Java Exception                                                                                   
from java.lang import Exception
from time import sleep

#
CCS.setThrowExceptions(True)

#time to wait for switch change
waitTime = int(5)

#time to do a nap
napTime = int(2)

#####################################################################
# launch loader-standalone subsystem and initialize
#####################################################################
loaderStandalone = CCS.attachSubsystem("loader-standalone")
print "Connected to loader-standalone ", loaderStandalone

#getting timeouts
loaderConfigFuture = loaderStandalone.synchCommandLine("getConfigurationInfo")
print "getting loader configuration info"
loaderConfig = loaderConfigFuture.get()
timeoutHandoff = loaderConfig.getCurrentValueForParameter("carrier//timeoutForGoingToHandOff")
timeoutStorage = loaderConfig.getCurrentValueForParameter("carrier//timeoutForGoingToStorage")
timeoutToOpen = loaderConfig.getCurrentValueForParameter("clamp//timeoutForOpeningHooks")

###loader
loader = CCS.attachSubsystem("loader-standalone/loader")

###carrier
carrier = CCS.attachSubsystem("loader-standalone/carrier")

##Clamp
clamp = CCS.attachSubsystem("loader-standalone/clamp")
hooksController = CCS.attachSubsystem("loader-standalone/hooksController")

###plutoGateway
plutoGateway = CCS.attachSubsystem("loader-standalone/loaderPlutoGateway")

###fake autochanger
autochanger = CCS.attachSubsystem("loader-standalone/autochanger")


##Proxy
tcpProxy = CCS.attachSubsystem("loader-standalone/loaderTcpProxy")

storagePosition=int((carrier.synchCommandLine(5, "getStoragePosition")).getResult())
print "STORAGE position=", storagePosition

handoffPosition=int((carrier.synchCommandLine(5, "getHandoffPosition")).getResult())
print "HANDOFF position=", handoffPosition

CCS.setThrowExceptions(True)

result = tcpProxy.synchCommandLine(5, "allDevicesBooted")
if result.getResult():
    print "allDevicesBooted=", result.getResult()
else:
    raise Exception("CANopen Hardware not ready")

############################################################################
# Check controllers and plutoGateway are READY
############################################################################
#hardware is READY 
#print "Check that controllers and plutoGateway are initialized...."
if plutoGateway.synchCommandLine(5, "isInitialized").getResult():
    print "plutoGateway.isInitialized()=", result.getResult()
else:
    raise Exception("plutoGateway not Ready")

if hooksController.synchCommandLine(5, "isInitialized").getResult():
    print "hooksController.isInitialized()=", result.getResult()
else:
    raise Exception("hooksController not Ready")

############################################################################
# Check initial conditions
############################################################################
#loader is connected on camera
if loader.synchCommandLine(5, "isConnectedOnCamera").getResult():
    print "loader carrier is connected on camera: OK"
else:
    raise Exception("loader carrier is not connected on camera")

############################################################################
# def go to initial conditions
############################################################################
def goToInitialConditions():
    print "BEGIN goToInitialConditions"
    print "Initial conditions : empty at STORAGE position..."
    #no filter is in the loader
    if loader.synchCommandLine(5, "isEmpty").getResult():
        print "loader is NOT holding a filter: OK"
        goToInitialConditionsEmpty()
    #a filter is in the loader 
    else:
        print "a filter is on the loader"
        goToInitialConditionsLoaded()
    print "END goToInitialConditions"

############################################################################
# def go to initial conditions when empty
############################################################################        
def goToInitialConditionsEmpty():
    #clamp should be opened
    if not clamp.synchCommandLine(5, "isOpened").getResult():
        print "clamp has to be open : opening clamp..."
        clamp.synchCommandLine(int(timeoutToOpen), "open")
    #carrier is at STORAGE : nothing to do
    if carrier.synchCommandLine(5, "isAtStoragePosition").getResult():
        print "loader carrier is at STORAGE position: OK"
    #carrier is NOT at STORAGE : have to move carrier to STORAGE
    else:
        ###go to STORAGE
        print "Loader going to STORAGE without filter ...."
        carrier.synchCommandLine(int(timeoutStorage), "goToStorage")        
            
############################################################################
# def go to initial conditions when a filter is in the loader
############################################################################   
def goToInitialConditionsLoaded():
    #clamp should be clamped
    if clamp.synchCommandLine(5, "isClamped").getResult():
        print "clamp is clamped on filter: OK"
    else:
        print "clamping...."
        clamp.synchCommandLine(int(timeoutToOpen), "clamp")
    #clamp is now clamped so carrier can be moved with filter
    sequenceGoToHandoffWithFilterUnclampAndOpen()
    sequenceGoToStorageWithoutFilter()
        
    

############################################################################
# def to wait for autochanger holding filter
############################################################################

def waitForAutochangerHoldingFilter():
    loaderStandalone.synchCommandLine(5, "updateStateWithSensors")
    while not autochanger.synchCommandLine(5, "isHoldingFilter").getResult():
        print "Waiting for autochanger hold filter (AP2 & AF3) ..."
        sleep(waitTime)
        loaderStandalone.synchCommandLine(5, "updateStateWithSensors")
    else:
        print "autochanger is holding filter"

############################################################################
# def to wait for autochanger NOT holding filter
############################################################################
def waitForAutochangerNOTHoldingFilter():
    loaderStandalone.synchCommandLine(5, "updateStateWithSensors")
    while autochanger.synchCommandLine(5, "isHoldingFilter").getResult():
        print "Waiting for autochanger release filter (AP2 & AF0 & AF1) ..."
        sleep(waitTime)
        loaderStandalone.synchCommandLine(5, "updateStateWithSensors")
        sleep(1)
    else:
        print "autochanger is NOT holding filter"


############################################################################
# def to do a little nap
############################################################################
def littleNap():
    ### a little nap to let time to see what's hapenning
    print "a little nap to let time to see what's hapenning"
    sleep(napTime)

################################################################################
# Sequence :
# - goToHandoff,
# - close,
# - wait for autochanger to release filter,
# - goToStorage
# - goToHandoff
# - unclamp,
# - open
################################################################################
CCS.setThrowExceptions(True)

###
def sequenceGoToHandoffWithoutFilterAndCloseClamp():

    print "=> sequenceGoToHandoffWithoutFilterAndCloseClamp"
    ###goToHandoff
    print "Going to handoff without filter...."
    carrier.synchCommandLine(int(timeoutHandoff), "goToHandoff")

    waitForAutochangerHoldingFilter()

    ###closing hooks
    print "Closing hooks at HANDOFF...."
    clamp.synchCommandLine(int(timeoutToOpen), "close")

###
def sequenceClampAndGoToStorageWithFilter():

    print "=> sequenceClampAndGoToStorageWithFilter"
    ###wait for autochanger not holding filter
    waitForAutochangerNOTHoldingFilter()

    ###clamping hooks
    print "Clamping hooks at HANDOFF...."
    clamp.synchCommandLine(int(timeoutToOpen), "clamp")

    littleNap()

    ###go to STORAGE
    print "Loader going to STORAGE with filter ...."
    carrier.synchCommandLine(int(timeoutStorage), "goToStorage")

    littleNap()

###
def sequenceGoToHandoffWithFilterUnclampAndOpen():

    print "=>   sequenceGoToHandoffWithFilterUnclampAndOpen"
    ###goToHandoff
    print "Going to handoff with filter...."
    carrier.synchCommandLine(int(timeoutHandoff), "goToHandoff")     

    ###unclamping hooks
    print "Unclamping hooks at HANDOFF ...."
    clamp.synchCommandLine(int(timeoutToOpen), "unclamp")

    waitForAutochangerHoldingFilter()

    ###opening hooks
    print "Opening hooks at HANDOFF...."
    clamp.synchCommandLine(int(timeoutToOpen), "open")

###
def sequenceGoToStorageWithoutFilter():

    print "=> sequenceGoToStorageWithoutFilter"
    
    ###go to STORAGE
    print "Loader going to STORAGE without filter ...."
    carrier.synchCommandLine(int(timeoutStorage), "goToStorage")


################################################################################
# Main Sequence
################################################################################s
goToInitialConditions()
for i in range(iter):
  
    print "begin ITERATION  ", i
    sequenceGoToHandoffWithoutFilterAndCloseClamp()
    sequenceClampAndGoToStorageWithFilter()
    sequenceGoToHandoffWithFilterUnclampAndOpen()
    sequenceGoToStorageWithoutFilter()
    print "end ITERATION  ", i
    
print "END LOOP"

==> org-lsst-ccs-subsystem-fcs-main-5.3.4-tests.jar: org/lsst/ccs/subsystems/fcs/scripts/loaderClampCarrierTest.py <==
#####################################################################
# To test sequence of actions on loader :
# goToHandoff, close, clamp, goToStorage, goToHandoff, unclamp, open
# Tested on loader proto at LPSC in May 2017
# In scripts directory (org-lsst-ccs-subsystem-fcs/main/scripts)
# this script's name is loaderClampCarrierTest.py
#####################################################################
### change iteration number / nombre d'iterations a changer si besoin est
iter = 2
#####################################################################

#import CCS Scripting Implementation                                                                                          
from org.lsst.ccs.scripting import CCS
#import the definition for a Java Exception                                                                                   
from java.lang import Exception
from time import sleep
import java.time
#
CCS.setThrowExceptions(True)

#time to wait for switch change
waitTime = int(5)

#time to do a nap
napTime = int(2)

#####################################################################
# launch loader-standalone subsystem and initialize
#####################################################################
loaderStandalone = CCS.attachSubsystem("loader-standalone")
print "Connected to loader-standalone ", loaderStandalone

#getting timeouts
loaderConfigFuture = loaderStandalone.sendSynchCommand("getConfigurationInfo")
print "getting loader configuration info"
loaderConfig = loaderConfigFuture.get()
timeoutHandoff = java.time.Duration.ofMillis(int(loaderConfig.getCurrentValueForParameter("carrier//timeoutForGoingToHandOff")))
timeoutStorage = java.time.Duration.ofMillis(int(loaderConfig.getCurrentValueForParameter("carrier//timeoutForGoingToStorage")))
timeoutToOpen = java.time.Duration.ofMillis(int(loaderConfig.getCurrentValueForParameter("clamp//timeoutForOpeningHooks")))
timeoutFiveMillis = java.time.Duration.ofMillis(5)

###loader
loader = CCS.attachSubsystem("loader-standalone/loader")

###carrier
carrier = CCS.attachSubsystem("loader-standalone/carrier")

##Clamp
clamp = CCS.attachSubsystem("loader-standalone/clamp")
hooksController = CCS.attachSubsystem("loader-standalone/hooksController")

###plutoGateway
plutoGateway = CCS.attachSubsystem("loader-standalone/loaderPlutoGateway")

###fake autochanger
autochanger = CCS.attachSubsystem("loader-standalone/autochanger")


##Proxy
tcpProxy = CCS.attachSubsystem("loader-standalone/loaderTcpProxy")

storagePosition=int((carrier.sendSynchCommand(timeoutFiveMillis, "getStoragePosition")))
print "STORAGE position=", storagePosition

handoffPosition=int((carrier.sendSynchCommand(timeoutFiveMillis, "getHandoffPosition")))
print "HANDOFF position=", handoffPosition

CCS.setThrowExceptions(True)

result = tcpProxy.sendSynchCommand(timeoutFiveMillis, "allDevicesBooted")
if result.getResult():
    print "allDevicesBooted=", result.getResult()
else:
    raise Exception("CANopen Hardware not ready")

############################################################################
# Check controllers and plutoGateway are READY
############################################################################
#hardware is READY 
#print "Check that controllers and plutoGateway are initialized...."
if plutoGateway.sendSynchCommand(timeoutFiveMillis, "isInitialized"):
    print "plutoGateway.isInitialized()=", result.getResult()
else:
    raise Exception("plutoGateway not Ready")

if hooksController.sendSynchCommand(timeoutFiveMillis, "isInitialized"):
    print "hooksController.isInitialized()=", result.getResult()
else:
    raise Exception("hooksController not Ready")

############################################################################
# Check initial conditions
############################################################################
#loader is connected on camera
if loader.sendSynchCommand(timeoutFiveMillis, "isConnectedOnCamera"):
    print "loader carrier is connected on camera: OK"
else:
    raise Exception("loader carrier is not connected on camera")

############################################################################
# def go to initial conditions
############################################################################
def goToInitialConditions():
    print "BEGIN goToInitialConditions"
    print "Initial conditions : empty at STORAGE position..."
    #no filter is in the loader
    if loader.sendSynchCommand(timeoutFiveMillis, "isEmpty"):
        print "loader is NOT holding a filter: OK"
        goToInitialConditionsEmpty()
    #a filter is in the loader 
    else:
        print "a filter is on the loader"
        goToInitialConditionsLoaded()
    print "END goToInitialConditions"

############################################################################
# def go to initial conditions when empty
############################################################################        
def goToInitialConditionsEmpty():
    #clamp should be opened
    if not clamp.sendSynchCommand(timeoutFiveMillis, "isOpened"):
        print "clamp has to be open : opening clamp..."
        clamp.sendSynchCommand(timeoutToOpen, "open")
    #carrier is at STORAGE : nothing to do
    if carrier.sendSynchCommand(timeoutFiveMillis, "isAtStoragePosition"):
        print "loader carrier is at STORAGE position: OK"
    #carrier is NOT at STORAGE : have to move carrier to STORAGE
    else:
        ###go to STORAGE
        print "Loader going to STORAGE without filter ...."
        carrier.sendSynchCommand(timeoutStorage, "goToStorage")        
            
############################################################################
# def go to initial conditions when a filter is in the loader
############################################################################   
def goToInitialConditionsLoaded():
    #clamp should be clamped
    if clamp.sendSynchCommand(timeoutFiveMillis, "isClamped"):
        print "clamp is clamped on filter: OK"
    else:
        print "clamping...."
        clamp.sendSynchCommand(timeoutToOpen, "clamp")
    #clamp is now clamped so carrier can be moved with filter
    sequenceGoToHandoffWithFilterUnclampAndOpen()
    sequenceGoToStorageWithoutFilter()
        
    

############################################################################
# def to wait for autochanger holding filter
############################################################################

def waitForAutochangerHoldingFilter():
    loaderStandalone.sendSynchCommand(timeoutFiveMillis, "updateStateWithSensors")
    while not autochanger.sendSynchCommand(timeoutFiveMillis, "isHoldingFilter"):
        print "Waiting for autochanger hold filter (AP2 & AF3) ..."
        sleep(waitTime)
        loaderStandalone.sendSynchCommand(timeoutFiveMillis, "updateStateWithSensors")
    else:
        print "autochanger is holding filter"

############################################################################
# def to wait for autochanger NOT holding filter
############################################################################
def waitForAutochangerNOTHoldingFilter():
    loaderStandalone.sendSynchCommand(timeoutFiveMillis, "updateStateWithSensors")
    while autochanger.sendSynchCommand(timeoutFiveMillis, "isHoldingFilter"):
        print "Waiting for autochanger release filter (AP2 & AF0 & AF1) ..."
        sleep(waitTime)
        loaderStandalone.sendSynchCommand(timeoutFiveMillis, "updateStateWithSensors")
        sleep(1)
    else:
        print "autochanger is NOT holding filter"


############################################################################
# def to do a little nap
############################################################################
def littleNap():
    ### a little nap to let time to see what's hapenning
    print "a little nap to let time to see what's hapenning"
    sleep(napTime)

################################################################################
# Sequence :
# - goToHandoff,
# - close,
# - wait for autochanger to release filter,
# - goToStorage
# - goToHandoff
# - unclamp,
# - open
################################################################################
CCS.setThrowExceptions(True)

###
def sequenceGoToHandoffWithoutFilterAndCloseClamp():

    print "=> sequenceGoToHandoffWithoutFilterAndCloseClamp"
    ###goToHandoff
    print "Going to handoff without filter...."
    carrier.sendSynchCommand(timeoutHandoff, "goToHandoff")

    waitForAutochangerHoldingFilter()

    ###closing hooks
    print "Closing hooks at HANDOFF...."
    clamp.sendSynchCommand(timeoutToOpen, "close")

###
def sequenceClampAndGoToStorageWithFilter():

    print "=> sequenceClampAndGoToStorageWithFilter"
    ###wait for autochanger not holding filter
    waitForAutochangerNOTHoldingFilter()

    ###clamping hooks
    print "Clamping hooks at HANDOFF...."
    clamp.sendSynchCommand(timeoutToOpen, "clamp")

    littleNap()

    ###go to STORAGE
    print "Loader going to STORAGE with filter ...."
    carrier.sendSynchCommand(timeoutStorage, "goToStorage")

    littleNap()

###
def sequenceGoToHandoffWithFilterUnclampAndOpen():

    print "=>   sequenceGoToHandoffWithFilterUnclampAndOpen"
    ###goToHandoff
    print "Going to handoff with filter...."
    carrier.sendSynchCommand(timeoutHandoff, "goToHandoff")     

    ###unclamping hooks
    print "Unclamping hooks at HANDOFF ...."
    clamp.sendSynchCommand(timeoutToOpen, "unclamp")

    waitForAutochangerHoldingFilter()

    ###opening hooks
    print "Opening hooks at HANDOFF...."
    clamp.sendSynchCommand(timeoutToOpen, "open")

###
def sequenceGoToStorageWithoutFilter():

    print "=> sequenceGoToStorageWithoutFilter"
    
    ###go to STORAGE
    print "Loader going to STORAGE without filter ...."
    carrier.sendSynchCommand(timeoutStorage, "goToStorage")


################################################################################
# Main Sequence
################################################################################s
goToInitialConditions()
for i in range(iter):
  
    print "begin ITERATION  ", i
    sequenceGoToHandoffWithoutFilterAndCloseClamp()
    sequenceClampAndGoToStorageWithFilter()
    sequenceGoToHandoffWithFilterUnclampAndOpen()
    sequenceGoToStorageWithoutFilter()
    print "end ITERATION  ", i
    
print "END LOOP"
